var fs = require('fs');
var path = require('path');
var handlebars = require('handlebars');
var GulpProject = require('./gulp-project');

var defaultTemplate = handlebars.compile(fs.readFileSync(
      path.resolve(__dirname, 'views', 'error.handlebars')).toString());

function errorResponse(response, log, template) {
  response.status(500);
  if(template) {
    response.render(template, log);
  } else {
    response.send(defaultTemplate(log));
  }
};

module.exports = function(src, watch, template) {
  var gp = new GulpProject(path.resolve(src));

  // Always kick off an initial build
  gp.build();

  // If we're supposed to, kick off a build whenever anything
  // changes.
  if(watch && fs.existsSync(watch)) {
    var watcher = fs.watch(watch, gp.build.bind(gp));
  }

  var griddle = function(request, response, next) {
    switch(gp.status()) {
      case GulpProject.Status.LAST_BUILD_FAILED:
        errorResponse(response, gp.log(), template);
        break;
      case GulpProject.Status.BUILDING:
        gp.buildCompleted().then(next, function() {
          response.status(500);
          errorResponse(response, gp.log(), template);
        });
        break;
      default:
        next();
    }
  };

  griddle.unwatch = function() {
    if(watcher) {
      watcher.close();
    }
  };

  return griddle;
};
