var util = require('util');
var events = require('events');

function SerialQueue() {
  this.reset();
}

util.inherits(SerialQueue, events.EventEmitter);

SerialQueue.prototype.push = function(fn) {
  if(this.fns.push(fn) === 1) {
    this.next();
  }
  return this;
};

SerialQueue.prototype.next = function(force) {
  if(this.status === SerialQueue.Status.STOPPED) {
    this.reset();
  }
  if(this.status !== SerialQueue.Status.EXECUTING || force) {
    var f = this.fns.shift();
    if(f) {
      this.status = SerialQueue.Status.EXECUTING;
      f().then(
        function() {
          this.log = this.log.concat(Array.prototype.slice.call(arguments));
          this.next(true);
        }.bind(this),
        function() {
          this.log = this.log.concat(Array.prototype.slice.call(arguments));
          this.stop.apply(this, Array.prototype.slice.call(arguments));
        }.bind(this)
      );
    } else {
      this.status = SerialQueue.Status.IDLE;
      this.emit(SerialQueue.Event.DONE, false);
    }
  }
};

SerialQueue.prototype.reset = function() {
  this.fns = [];
  this.log = [];
  this.status = SerialQueue.Status.IDLE;
  return this;
};

SerialQueue.prototype.stop = function() {
  this.status = SerialQueue.Status.STOPPED;
  this.emit.apply(this, [ SerialQueue.Event.DONE, true ].concat(
      Array.prototype.slice.call(arguments)));
};

SerialQueue.Event = {
  DONE: 'done'
};
Object.freeze(SerialQueue.Event);

SerialQueue.Status = {
  EXECUTING: 'executing',
  STOPPED: 'stopped',
  IDLE: 'idle'
};
Object.freeze(SerialQueue.Status);

module.exports = SerialQueue;
